@@ -5,7 +5,7 @@
  *
  * See http://opensource.adnovum.ch/mod_qos/ for further details.
  *
- * Copyright (C) 2011-2013 Pascal Buchbinder
+ * Copyright (C) 2011-2014 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,11 +27,11 @@
  *
  */
 
-static const char revision[] = "$Id: qsgrep.c,v 1.13 2013/11/27 07:09:06 pbuchbinder Exp $";
+static const char revision[] = "$Id: qsgrep.c,v 1.15 2014/01/09 08:13:07 pbuchbinder Exp $";
 
 /* system */
 #include <stdio.h>
-#include <error.h>
+#include <errno.h>
 #include <string.h>
 
 #include <stdlib.h>
