@@ -9,7 +9,7 @@
  * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2007-2013 Pascal Buchbinder
+ * Copyright (C) 2007-2014 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,10 +28,9 @@
  *
  */
 
-static const char revision[] = "$Id: qslog.c,v 1.71 2013/11/27 07:09:06 pbuchbinder Exp $";
+static const char revision[] = "$Id: qslog.c,v 1.83 2014/01/09 08:13:07 pbuchbinder Exp $";
 
 #include <stdio.h>
-#include <error.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <errno.h>
@@ -70,6 +69,16 @@ static const char revision[] = "$Id: qslog.c,v 1.71 2013/11/27 07:09:06 pbuchbin
  * structures
  * ---------------------------------- */
 
+typedef struct {
+  long request_count;
+  long status_1;
+  long status_2;
+  long status_3;
+  long status_4;
+  long status_5;
+  long long duration_count_ms;
+} url_rec_t;
+
 typedef struct {
   long request_count;
   long error_count;
@@ -169,6 +178,7 @@ static int   m_generations = QS_GENERATIONS;
 static regex_t m_trx;
 static regex_t m_trx2;
 /* real time mode (default) or offline */
+static int   m_off = 0;
 static int   m_offline = 0;
 static int   m_offline_data = 0;
 static char  m_date_str[MAX_LINE];
@@ -177,8 +187,11 @@ static int   m_avms = 0;
 static int   m_ct = 0;
 static int   m_customcounter = 0;
 static apr_table_t *m_client_entries = NULL;
-static int   m_max_client_entries = 0;
+static int   m_max_entries = 0;
 static int   m_offline_count = 0;
+static apr_table_t *m_url_entries = NULL;
+static int   m_offline_url = 0;
+static int   m_offline_url_cropped = 0;
 static int   m_methods = 0;
 /* debug/offline */
 static long  m_lines = 0;
@@ -251,7 +264,7 @@ static char *skipElement(const char* line) {
     p++;
   } else {
     char *eq = NULL;
-    if(m_offline || m_offline_count) {
+    if(m_off) {
       // offline mode: check for <name>='<value>' entry
       eq = strstr(p, "='");
       if(eq && (eq - p) < 10) {
@@ -375,7 +388,14 @@ static void getFreeMem(char *buf, int sz) {
     fclose(f);
     snprintf(buf, sz, "%d", mem);
   } else {
-    /* experimental code using vmstat */
+    // non linux
+//#ifdef _SC_AVPHYS_PAGES
+//    long pageSize = sysconf(_SC_PAGESIZE);
+//    long freePages = sysconf(_SC_AVPHYS_PAGES);
+//    mem = pageSize * freePages / 1024;
+//    snprintf(buf, sz, "%d", mem);
+//#else
+    /* fallback using vmstat (experimental code) */
     char vmstat[] = "/usr/bin/vmstat";
     struct stat attr;
     if(stat(vmstat, &attr) == 0) {
@@ -390,7 +410,7 @@ static void getFreeMem(char *buf, int sz) {
         int i = 1;
         while(!qs_getLinef(line, sizeof(line), f)) {
           if(i == 4) {
-            // free memory only (ignores cache on linux)
+            // free memory only (ignores cache)
             int j = 0;
             char *p = line;
             while(p && j < 4) {
@@ -415,6 +435,7 @@ static void getFreeMem(char *buf, int sz) {
         unlink(outfile);
       }
     }
+//#endif
   }
 }
 
@@ -577,10 +598,11 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
   stat_rec->qos_ser = 0;
   if(main) {
     if(!offline) {
-      fprintf(f, "sl;%.2f;m;%s;",
-              av[0], mem[0] ? mem : "-");
+      fprintf(f, "sl;%.2f;", av[0]);
+      if(m_mem) {
+        fprintf(f, "m;%s;", mem[0] ? mem : "-");
+      }
     } else {
-      fprintf(f, "sl;-;m;-;");
       m_offline_data = 0;
     }
   }
@@ -784,6 +806,77 @@ static void printAndResetStat(char *timeStr) {
   }
 }
 
+/**
+ * Updates the per url records
+ */
+static void updateUrl(apr_pool_t *pool, char *R, char *S, long tmems) {
+  url_rec_t *url_rec;
+  char *marker;
+  if(R == NULL) {
+    return;
+  }
+  if(!isalpha(R[0])) {
+    fprintf(stdout, "A(%ld)", m_lines);
+    return;
+  }
+  marker = strchr(R, ' ');
+  if(marker == NULL) {
+    fprintf(stdout, "E(%ld)", m_lines);
+    return;
+  }
+  marker[0] = ';';
+  marker = strrchr(R, ' ');
+  if(marker) {
+    marker[0] = '\0';
+  }
+  marker = strchr(R, '?');
+  if(marker) {
+    marker[0] = '\0';
+  }
+  if(m_offline_url_cropped) {
+    char *root = strchr(R, '/');
+    marker = strrchr(R, '/');
+    if(marker && marker != root) {
+      marker[0] = '\0';
+    }
+  }
+  url_rec = (url_rec_t *)apr_table_get(m_url_entries, R);
+  if(url_rec == NULL) {
+    if(apr_table_elts(m_url_entries)->nelts >= MAX_CLIENT_ENTRIES) {
+      // limitation
+      if(!m_max_entries) {
+        fprintf(stderr, "\nreached max url entries (%d)\n", MAX_CLIENT_ENTRIES);
+        m_max_entries = 1;
+      }
+      return;
+    }
+    url_rec = apr_pcalloc(pool, sizeof(url_rec_t));
+    url_rec->request_count = 0;
+    url_rec->status_1 = 0;
+    url_rec->status_2 = 0;
+    url_rec->status_3 = 0;
+    url_rec->status_4 = 0;
+    url_rec->status_5 = 0;
+    url_rec->duration_count_ms = 0;
+    apr_table_setn(m_url_entries, apr_pstrdup(pool, R), (char *)url_rec);
+  }
+  url_rec->request_count++;
+  if(S[0] == '1') {
+    url_rec->status_1++;
+  } else if(S[0] == '1') {
+    url_rec->status_1++;
+  } else if(S[0] == '2') {
+    url_rec->status_2++;
+  } else if(S[0] == '3') {
+    url_rec->status_3++;
+  } else if(S[0] == '4') {
+    url_rec->status_4++;
+  } else if(S[0] == '5') {
+    url_rec->status_5++;
+  }
+  url_rec->duration_count_ms += tmems;
+}
+
 /**
  * Updates the per client record
  */
@@ -804,9 +897,9 @@ static void updateClient(apr_pool_t *pool, char *T, char *t, char *D, char *S,
     char *tid;
     if(apr_table_elts(m_client_entries)->nelts >= MAX_CLIENT_ENTRIES) {
       // limitation: speed (table to big) and memory
-      if(!m_max_client_entries) {
-        fprintf(stderr, "\nreached max client enries (%d)\n", MAX_CLIENT_ENTRIES);
-        m_max_client_entries = 1;
+      if(!m_max_entries) {
+        fprintf(stderr, "\nreached max client entries (%d)\n", MAX_CLIENT_ENTRIES);
+        m_max_entries = 1;
       }
       return;
     }
@@ -1064,7 +1157,7 @@ static void updateStat(apr_pool_t *pool, const char *cstr, char *line) {
   long tme;
   long tmems;
   if(!line[0]) return;
-  if(m_offline || m_offline_count) {
+  if(m_off) {
     m_lines++;
   }
   while(c[0]) {
@@ -1204,25 +1297,33 @@ static void updateStat(apr_pool_t *pool, const char *cstr, char *line) {
       tme = tmems / 1000;
     }
   }
-  if(!m_offline_count) {
+  if(m_offline_count) {
+    updateClient(pool, T, t, D, S, BI, B, R, I, U, Q, E, k, C, ct, tme, tmems, m);
+  } else if(m_offline_url) {
+    if((tmems) == 0 && (tme > 0)) {
+      tmems = 1000 * tme;
+    }
+    updateUrl(pool, R, S, tmems);
+  } else {
     updateRec(m_stat_rec, T, t, D, S, s, a, A, BI, B, R, I, U, Q, E, k, C, tme, tmems);
     if(rec) {
       updateRec(rec, T, t, D, S, s, a, A, BI, B, R, I, U, Q, E, k, C, tme, tmems);
     }
-  } else {
-    updateClient(pool, T, t, D, S, BI, B, R, I, U, Q, E, k, C, ct, tme, tmems, m);
   }
   qs_csUnLock();
 
-  if(m_verbose && (m_offline || m_offline_count)) {
-    printf("[%ld] I=[%s] U=[%s] B=[%s] i=[%s] S=[%s] T=[%ld] Q=[%s]\n", m_lines,
+  if(m_verbose && m_off) {
+    printf("[%ld] I=[%s] U=[%s] B=[%s] i=[%s] S=[%s] T=[%ld](%ld) Q=[%s] E=[%s] k=[%s] R=[%s]\n", m_lines,
            I == NULL ? "(null)" : I,
            U == NULL ? "(null)" : U,
            B == NULL ? "(null)" : B,
            BI == NULL ? "(null)" : BI,
            S == NULL ? "(null)" : S,
-           tme,
-           Q == NULL ? "(null)" : Q
+           tme, tmems,
+           Q == NULL ? "(null)" : Q,
+           E == NULL ? "(null)" : E,
+           k == NULL ? "(null)" : k,
+           R == NULL ? "(null)" : R
            );
   }
   line[0] = '\0';
@@ -1287,7 +1388,7 @@ static time_t getMinutes(char *line) {
       return minutes;
     } else {
       // unknown format
-      fprintf(stdout, "E");
+      fprintf(stdout, "F(%ld)", m_lines);
       return 0;
     }
   } else {
@@ -1359,7 +1460,7 @@ static void readStdinOffline(apr_pool_t *pool, const char *cstr) {
   time_t unitTime = 0;
   int line_len;
   FILE *outdev = stdout;
-  if(m_offline_count) {
+  if(m_offline_count || m_offline_url) {
     outdev = stderr;
   }
   while(fgets(line, sizeof(line), stdin) != NULL) {
@@ -1475,7 +1576,7 @@ static void usage(const char *cmd, int man) {
   if(man) {
     printf(".SH SYNOPSIS\n");
   }
-  qs_man_print(man, "%s%s -f <format_string> -o <out_file> [-p[c] [-v]] [-x [<num>]] [-u <name>] [-m] [-c <path>]\n", man ? "" : "Usage: ", cmd);
+  qs_man_print(man, "%s%s -f <format_string> -o <out_file> [-p[c|u[c]] [-v]] [-x [<num>]] [-u <name>] [-m] [-c <path>]\n", man ? "" : "Usage: ", cmd);
   printf("\n");
   if(man) {
     printf(".SH DESCRIPTION\n");
@@ -1486,7 +1587,7 @@ static void usage(const char *cmd, int man) {
   qs_man_print(man, "the data from stdin. The output is written to the specified\n");
   qs_man_println(man, "file every minute and includes the following entries:\n");
   qs_man_println(man, "  - requests per second ("NRS")\n");
-  qs_man_println(man, "  - requests within the last minute (req)\n");
+  qs_man_println(man, "  - number of requests within measured time (req)\n");
   qs_man_println(man, "  - bytes sent to the client per second ("NBS")\n");
   qs_man_println(man, "  - bytes received from the client per second ("NBIS")\n");
   qs_man_println(man, "  - repsonse status codes within the last minute (1xx,2xx,3xx,4xx,5xx)\n");
@@ -1495,7 +1596,7 @@ static void usage(const char *cmd, int man) {
   qs_man_println(man, "  - distribution of response durations within the last minute\n");
   qs_man_print(man, "    (<1s,1s,2s,3s,4s,5s,>5)\n");
   if(man) printf("\n");
-  qs_man_println(man, "  - number of established (new) connections within the last minutes (esco)\n");
+  qs_man_println(man, "  - number of established (new) connections within the measured time (esco)\n");
   qs_man_println(man, "  - average system load (sl)\n");
   qs_man_println(man, "  - free memory (m) (not available for all platforms)\n");
   qs_man_println(man, "  - number of client ip addresses seen withn the last %d seconds (ip)\n", ACTIVE_TIME);
@@ -1531,9 +1632,9 @@ static void usage(const char *cmd, int man) {
   qs_man_println(man, "     U defines the user tracking id (%%{mod_qos_user_id}e)\n");
   qs_man_println(man, "     Q defines the mod_qos_ev event message (%%{mod_qos_ev}e)\n");
   qs_man_println(man, "     C defines the element for the detailed log (-c option), e.g. \"%%U\"\n");
-  qs_man_println(man, "     s arbitraty counter to add up (sum within a minute)\n");
-  qs_man_println(man, "     a arbitraty counter to build an average from (average per request)\n");
-  qs_man_println(man, "     A arbitraty counter to build an average from (average per request)\n");
+  qs_man_println(man, "     s arbitrary counter to add up (sum within a minute)\n");
+  qs_man_println(man, "     a arbitrary counter to build an average from (average per request)\n");
+  qs_man_println(man, "     A arbitrary counter to build an average from (average per request)\n");
   qs_man_println(man, "     E comma separated list of event strings\n");
   qs_man_println(man, "     c content type (%%{content-type}o), available in -pc mode only\n");
   qs_man_println(man, "     m request method (GET/POST) (%%m), available in -pc mode only\n");
@@ -1546,12 +1647,15 @@ static void usage(const char *cmd, int man) {
   qs_man_print(man, "  -p\n");
   if(man) printf("\n");
   qs_man_print(man, "     Used for post processing when reading the log data from a file (cat/pipe).\n");
-  qs_man_print(man, "     %s is started using it's offline mode in order to process existing log\n", cmd);
-  qs_man_print(man, "     files (post processing).\n");
+  qs_man_print(man, "     %s is started using it's offline mode (extracting the time stamps from\n", cmd);
+  qs_man_print(man, "     the log lines) in order to process existing log files.\n");
   qs_man_print(man, "     The option \"-pc\" may be used alternatively if you want to gather request\n");
   qs_man_print(man, "     information per client (identified by IP address (I) or user tracking id (U)\n");
   qs_man_print(man, "     showing how many request each client has performed within the captured period\n");
   qs_man_print(man, "     of time). \"-pc\" supports the format characters IURSBTtDkEcm.\n");
+  qs_man_print(man, "     The option \"-pu\" collects statistics on a per URL level (supports format\n");
+  qs_man_print(man, "     characters RSTtD).\n");
+  qs_man_print(man, "     \"-puc\" is very similar to \"-pu\" but cuts the end (handler) of each URL.\n");
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -v\n");
   if(man) printf("\n");
@@ -1726,12 +1830,19 @@ int main(int argc, const char *const argv[]) {
       if (--argc >= 1) {
         confFile = *(++argv);
       }
-    } else if(strcmp(*argv,"-p") == 0) { /* activate offline analysis */
+    } else if(strcmp(*argv,"-p") == 0) {   /* activate offline analysis */
       m_offline = 1;
       qs_set2OfflineMode();
-    } else if(strcmp(*argv,"-pc") == 0) { /* activate offline counting analysis */
+    } else if(strcmp(*argv,"-pc") == 0) {  /* activate offline counting analysis */
       m_offline_count = 1;
       qs_set2OfflineMode();
+    } else if(strcmp(*argv,"-pu") == 0) {  /* activate offline url analysis */
+      m_offline_url = 1;
+      qs_set2OfflineMode();
+    } else if(strcmp(*argv,"-puc") == 0) { /* activate offline url analysis */
+      m_offline_url = 1;
+      m_offline_url_cropped = 1;
+      qs_set2OfflineMode();
     } else if(strcmp(*argv,"-m") == 0) { /* activate memory usage */
       m_mem = 1;
     } else if(strcmp(*argv,"-v") == 0) {
@@ -1760,8 +1871,11 @@ int main(int argc, const char *const argv[]) {
     argc--;
     argv++;
   }
-
-  if(m_offline || m_offline_count) {
+  m_off = m_offline || m_offline_count || m_offline_url;
+  if(m_off) {
+    if(nice(10) == -1) {
+      fprintf(stderr, "ERROR, failed to change nice value: %s\n", strerror(errno));
+    }
     /* init time pattern regex, std apache access log */
     regcomp(&m_trx, 
             "[0-9]{2}/[a-zA-Z]{3}/[0-9]{4}:[0-9]{2}:[0-9]{2}:[0-9]{2}",
@@ -1772,6 +1886,86 @@ int main(int argc, const char *const argv[]) {
             REG_EXTENDED);
   }
 
+  /*
+   * offline url mod
+   */
+  if(m_offline_url) {
+    int i;
+    apr_table_entry_t *entry;
+    long request_count = 0;
+    long request_count_min = -1;
+    long request_count_max = 0;
+    long long duration_count_ms = 0;
+    long long duration_count_ms_min = -1;
+    long long duration_count_ms_max = 0;
+    m_url_entries = apr_table_make(pool, MAX_CLIENT_ENTRIES + 1);
+    readStdinOffline(pool, config);
+    fprintf(stderr, ".\n");
+
+    m_f = stdout;
+    if(file) {
+      m_f = fopen(file, "a+");
+      if(!m_f) {
+        m_f = stdout;
+      }
+    }
+    entry = (apr_table_entry_t *) apr_table_elts(m_url_entries)->elts;
+    for(i = 0; i < apr_table_elts(m_url_entries)->nelts; i++) {
+      url_rec_t *url_rec = (url_rec_t *)entry[i].val;
+      request_count += url_rec->request_count;
+      if(request_count_min == -1) {
+        request_count_min = url_rec->request_count;
+      }
+      if(request_count_min > url_rec->request_count) {
+        request_count_min = url_rec->request_count;
+      }
+      if(request_count_max < url_rec->request_count) {
+        request_count_max = url_rec->request_count;
+      }
+      duration_count_ms += url_rec->duration_count_ms;
+      if(duration_count_ms_min == -1) {
+        duration_count_ms_min = url_rec->duration_count_ms;
+      }
+      if(duration_count_ms_min > url_rec->duration_count_ms) {
+        duration_count_ms_min = url_rec->duration_count_ms;
+      }
+      if(duration_count_ms_max < url_rec->duration_count_ms) {
+        duration_count_ms_max = url_rec->duration_count_ms;
+      }
+      fprintf(m_f, "req;%ld;"
+              "1xx;%ld;2xx;%ld;3xx;%ld;4xx;%ld;5xx;%ld;"
+              NAVMS";%lld;%s\n",
+              url_rec->request_count,
+              url_rec->status_1,
+              url_rec->status_2,
+              url_rec->status_3,
+              url_rec->status_4,
+              url_rec->status_5,
+              url_rec->duration_count_ms / url_rec->request_count,
+              entry[i].key);
+      
+    }
+    fprintf(m_f, "req;%ld;"
+            ";;;;;;;;;;"
+            NAVMS";%lld;min;\n",
+            request_count_min,
+            duration_count_ms_min);
+    fprintf(m_f, "req;%ld;"
+            ";;;;;;;;;;"
+            NAVMS";%lld;max;\n",
+            request_count_max,
+            duration_count_ms_max);
+    fprintf(m_f, "req;%ld;"
+            ";;;;;;;;;;"
+            NAVMS";%lld;average;\n",
+            request_count / apr_table_elts(m_url_entries)->nelts,
+            duration_count_ms / request_count);
+    if(file && m_f != stdout) {
+      fclose(m_f);
+    }
+    return 0;
+  }
+
   /*
    * offline count mode creates statistics
    * on a per client basis (e.g. per source
@@ -1781,11 +1975,8 @@ int main(int argc, const char *const argv[]) {
   if(m_offline_count) {
     int i;
     apr_table_entry_t *entry;
-    if(nice(10) == -1) {
-      fprintf(stderr, "ERROR, failed to change nice value: %s\n", strerror(errno));
-    }
     if(config == NULL) usage(cmd, 0);
-    m_client_entries = apr_table_make(pool, MAX_CLIENT_ENTRIES);
+    m_client_entries = apr_table_make(pool, MAX_CLIENT_ENTRIES + 1);
     readStdinOffline(pool, config);
     fprintf(stderr, ".\n");
     entry = (apr_table_entry_t *) apr_table_elts(m_client_entries)->elts;
@@ -1934,9 +2125,6 @@ int main(int argc, const char *const argv[]) {
    * the date string match of the log
    * enties. */
   if(m_offline) {
-    if(nice(10) == -1) {
-      fprintf(stderr, "ERROR, failed to change nice value: %s\n", strerror(errno));
-    }
     fprintf(stderr, "[%s]: offline mode (writes to %s)\n", cmd, file);
     m_date_str[0] = '\0';
     readStdinOffline(pool, config);
